function [F G] = RobustSyncControl(A, B, C, L, N, eta, gamma, ssigma, tau)
% [F G] = RobustSyncControl(A, B, C, L, N, eta, gamma)
% Returns the gains F and G given by (33) and (34)
% 
% [F G] = RobustSyncControl(A, B, C, L, N, eta, gamma, sigma)
% [F G] = RobustSyncControl(A, B, C, L, N, eta, gamma, sigma, tau)
% The optional arguments 'sigma' and 'tau' force AQ+QA'-QC'CQ = - tau I thus making
% sure the condition AQ+QA'-QC'CQ < 0 is met. (defaults to tau=10^-10)
% and A'P + PA - gamma PBB'P = - sigma I making computation easier if A has (close to)
% purely imaginary eigenvalues (defaults to sigma=10^-10)
  
  % compute the eigenvalues of the graph laplacian.
  E = sort(real(eig(L)));
  lambda2 = E(2);
  lambdaP = E(end);
  
  % test for suitable choices of N and gamma (eq (30) and (31))
  if N <= lambdaP^2/lambda2
    warning('N out of bounds');
  end
  if ~(( lambdaP^2/N^2 <= gamma) && (gamma  < lambda2/N))
    warning('gamma out of bounds');
  end
  
  % setting ssigma, tau
  if nargin < 8
    tau = 10^-10;
  end
  if nargin < 9
    ssigma = 10^-10;
  end
  % calculate P and Q as given by (28) and (29)
  P = care(A, B, ssigma*eye(size(A)), 1/gamma*eye(size(B,2)));
  Q = care(A', C', tau*eye(size(A')));
  
  % test if our eta is a good choice (eq 32)
  if eta >= 1/sqrt(max(abs(eig(P*Q))))
    warning('eta out of bounds');
  end
  
  % calculate F and G (eq (33) and (34))
  F = -B'*P;
  G = inv(eye(size(Q*P)) - eta^2*Q*P) *Q*C';
end