%creates a small simulink model to test robust formation controll.
%2 double integators are used to simulate acceleration --> speed --> position of of the agent in the x and y direction.

%individual agent dynamics
A = [0 1 0 0; 0 0 0 0; 0 0 0 1; 0 0 0 0];
B = [0 0; 1 0; 0 0; 0 1];
C = [1 0 0 0; 0 0 1 0];

%uncomment to select the network topology and number of agents
%Graph=CompleteGraph(6);
Graph=CircleGraph(15);
%Graph=StarGraph(6); %star graph

L = GraphLaplacian(Graph);

%some useful numbers
n=size(A,1);
m=size(B,2);
q=size(C,1);
p = size(L,1);

StopTime = '8000'; %time to run the simulation
OutputTimes = mat2str(0:8:8000); %points that should be saved

%initial condition
X0=zeros(2*p*n,1);
X0(1:2:p*n)=2*rand(p*n/2,1)-1;

xy = [cos(0:2*pi/size(L,1):2*pi-2*pi/size(L,1))', sin(0:2*pi/size(L,1):2*pi-2*pi/size(L,1))']; %coordinates of the nodes (on a circle)
Formation = reshape(xy', 1, numel(xy))';

%Compute the protocol
[N, eta, gamma] = RobustSyncOptimalVars(A, B, C, L);
[F G] = RobustSyncControl(A, B, C, L, N, eta, gamma);

%compute the added perturbations
pertsys = perturbations(p, m, q, eta);

%compute the overal network dynamics
I = eye(p);
SSA = [ kron( I, A),           kron( I, B * F);
        1/N * kron( L, G * C), kron( I, A - G * C) + kron( 1/N * L, B * F)];
SSB = [zeros( p * size( G)) zeros( p * size( G)); kron( 1/N * L, G) kron( -1/N * L, G)];
SSC = [zeros( p * size( F)), kron( I, F)];

%build the simulink system:
sys = 'Model';
bdclose(sys)
new_system(sys) % Create the model
open_system(sys) % Open the model

%some constants defining the spacing in simulink
x = 50;
y = 50;
w = 50;
h = 50;
offset = 100;

%adding blocks and links
pos = [(x+offset*1) (y+offset*0) (x+offset*1)+w (y+offset*0)+h];
add_block('built-in/State-Space',[sys '/SS'],'Position',pos);

pos = [(x+offset*2) (y+offset*0) (x+offset*2)+w (y+offset*0)+h];
add_block('built-in/Gain',[sys '/Xstate'],'Position',pos);

pos = [(x+offset*2) (y+offset*1) (x+offset*2)+w (y+offset*1)+h];
add_block('built-in/Gain',[sys '/Z'],'Position',pos);

pos = [(x+offset*1) (y+offset*1) (x+offset*1)+w (y+offset*1)+h];
add_block('built-in/State-Space',[sys '/Perturbation'],'Position',pos);

pos = [(x+offset*3) (y+offset*0) (x+offset*3)+w (y+offset*0)+h];
add_block('built-in/Gain',[sys '/Cx'],'Position',pos);

pos = [(x+offset*3) (y+offset*1) (x+offset*3)+w (y+offset*1)+h];
add_block('built-in/Outport',[sys '/Out1'],'Position',pos);

pos = [(x+offset*0) (y+offset*1) (x+offset*0)+w (y+offset*1)+h];
add_block('built-in/Constant',[sys '/Formation'],'Position',pos);

pos = [(x+offset*0) (y+offset*0) (x+offset*0)+w (y+offset*0)+h];
add_block('built-in/Mux',[sys '/SSinput'],'Position',pos);

add_line(sys,'SS/1','Xstate/1','autorouting','on');
add_line(sys,'SS/1','Z/1','autorouting','on');
add_line(sys,'Z/1','Perturbation/1','autorouting','on');
add_line(sys,'Perturbation/1','SSinput/1','autorouting','on');
add_line(sys,'Xstate/1','Cx/1','autorouting','on');
add_line(sys,'Cx/1','Out1/1','autorouting','on');
add_line(sys,'SSinput/1','SS/1','autorouting','on');
add_line(sys,'Formation/1','SSinput/2','autorouting','on');

%adding parameters
set_param([sys '/SS'], 'A', mat2str(SSA), 'B', mat2str(SSB), 'C', mat2str(eye(size(SSA))), 'D', mat2str(zeros(size(SSB))), 'X0', mat2str(X0));
set_param([sys '/Z'], 'Multiplication', 'Matrix(K*u)', 'Gain', mat2str(SSC));
set_param([sys '/Perturbation'], 'A', mat2str(pertsys.A), 'B', mat2str(pertsys.B), 'C', mat2str(pertsys.C), 'D', mat2str(pertsys.D), 'X0', mat2str(zeros(size(pertsys.A,1),1)));
set_param([sys '/Xstate'], 'Multiplication', 'Matrix(K*u)', 'Gain', mat2str([eye(size(A)*p), zeros(size(A)*p)]));
set_param([sys '/Cx'], 'Multiplication', 'Matrix(K*u)', 'Gain', mat2str(kron(I,C)));
set_param([sys '/SSinput'], 'Inputs', '2');
set_param([sys '/Formation'], 'Value', mat2str(Formation));
set_param(sys, 'StopTime', StopTime, 'OutputOption', 'SpecifiedOutputTimes', 'OutputTimes', OutputTimes);

%run the simulation
sim(sys);

