%create a small simulink model to test if a system of integrators synchronises:

%%individual agent dynamics (double integrator)
A = [0.1 1; 0 0.1]; B = [0; 1]; C = [1 0];

%uncomment to select the network topology and number of agents
Graph=CompleteGraph(5);
%Graph=CircleGraph(9);
%Graph=StarGraph(7); %star graph

L = GraphLaplacian(Graph);

%some useful numbers
n=size(A,1);
m=size(B,2);
q=size(C,1);
p = size(L,1);

StopTime = '300'; %time to run the simulation
OutputTimes = mat2str(0:0.3:300); %points that should be saved

%setting up the initial condition(w(t=0) is left 0, x(t=0) is seeded with random numbers (-1,1))
X0=zeros(2*p*n,1);
X0(1:p*n)=2*rand(p*n,1)-1;

%Compute the protocol
[N, eta, gamma] = RobustSyncOptimalVars(A, B, C, L);
[F G] = RobustSyncControl(A, B, C, L, N, eta, gamma);

%compute the added perturbations
pertsys = perturbations(p, m, q, 0*eta);

%compute the overal network dynamics
I = eye(p);
SSA = [ kron( I, A),           kron( I, B * F);
        1/N * kron( L, G * C), kron( I, A - G * C) + kron( 1/N * L, B * F)];
SSB = [zeros( p * size( G)); kron( 1/N * L, G)];
SSC = [zeros( p * size( F)), kron( I, F)];

%build the simulink system:
sys = 'Model';
bdclose(sys)
new_system(sys) % Create the model
open_system(sys) % Open the model

%some constants defining the spacing in simulink
x = 50;
y = 50;
w = 50;
h = 50;
offset = 100;

%adding blocks and links
pos = [(x+offset*0) (y+offset*0) (x+offset*0)+w (y+offset*0)+h];
add_block('built-in/State-Space',[sys '/SS'],'Position',pos);

pos = [(x+offset*1) (y+offset*0) (x+offset*1)+w (y+offset*0)+h];
add_block('built-in/Gain',[sys '/Xstate'],'Position',pos);

pos = [(x+offset*1) (y+offset*1) (x+offset*1)+w (y+offset*1)+h];
add_block('built-in/Gain',[sys '/Z'],'Position',pos);

pos = [(x+offset*0) (y+offset*1) (x+offset*0)+w (y+offset*1)+h];
add_block('built-in/State-Space',[sys '/Perturbation'],'Position',pos);

pos = [(x+offset*2) (y+offset*0) (x+offset*2)+w (y+offset*0)+h];
add_block('built-in/Scope',[sys '/Scope1'],'Position',pos);

pos = [(x+offset*2) (y+offset*1) (x+offset*2)+w (y+offset*1)+h];
add_block('built-in/Gain',[sys '/Cx'],'Position',pos);

pos = [(x+offset*2) (y+offset*2) (x+offset*2)+w (y+offset*2)+h];
add_block('built-in/Outport',[sys '/Out1'],'Position',pos);

add_line(sys,'SS/1','Xstate/1','autorouting','on');
add_line(sys,'Xstate/1','Scope1/1','autorouting','on');
add_line(sys,'SS/1','Z/1','autorouting','on');
add_line(sys,'Z/1','Perturbation/1','autorouting','on');
add_line(sys,'Perturbation/1','SS/1','autorouting','on');
add_line(sys,'Xstate/1','Cx/1','autorouting','on');
add_line(sys,'Cx/1','Out1/1','autorouting','on');

%adding parameters
set_param([sys '/SS'], 'A', mat2str(SSA), 'B', mat2str(SSB), 'C', mat2str(eye(size(SSA))), 'D', mat2str(zeros(size(SSB))), 'X0', mat2str(X0));
set_param([sys '/Z'], 'Multiplication', 'Matrix(K*u)', 'Gain', mat2str(SSC));
set_param([sys '/Perturbation'], 'A', mat2str(pertsys.A), 'B', mat2str(pertsys.B), 'C', mat2str(pertsys.C), 'D', mat2str(pertsys.D), 'X0', mat2str(zeros(size(pertsys.A,1),1)));
set_param([sys '/Xstate'], 'Multiplication', 'Matrix(K*u)', 'Gain', mat2str([eye(size(A)*p), zeros(size(A)*p)]));
set_param([sys '/Cx'], 'Multiplication', 'Matrix(K*u)', 'Gain', mat2str(kron(I,C)));
set_param(sys, 'StopTime', StopTime, 'OutputOption', 'SpecifiedOutputTimes', 'OutputTimes', OutputTimes);
%enable simulink accelerator mode. Requires a compatible C compiler. Run 'mex -setup' if matlab isn't yet configured
%set_param(sys, 'SimulationMode', 'accelerator');

%run the simulation
sim(sys);

%plotting the output:
fig=figure;
subplot(2,2,1);
plot1=semilogy(tout, abs(yout));
title('Output');

%plot the difference between each node and the average
subplot(2,2,2);
plot2=plot(tout, bsxfun(@minus,yout,mean(yout')'));
title('Difference from average');

%plot the graph
subplot(2,2,3);
%put the nodes equally spaced on a circle
xy = [cos(0:2*pi/size(L,1):2*pi-2*pi/size(L,1))', sin(0:2*pi/size(L,1):2*pi-2*pi/size(L,1))'];
gplot(Graph, xy,'-*');
axis off;
for j = 1:p,
  text(xy(j,1), xy(j,2), [' ' int2str(j)], 'FontSize',12);
end
title('Graph');

%add a legend
sh=subplot(2,4,7);
s = cell(1, floor(p/2));
for i = 1:p,
  s{i} =  sprintf('Agent %d', i);
end
leg1=legend(sh,plot1(1:floor(p/2)),s{1:floor(p/2)});
set(leg1,'position', get(sh,'position'));
title(sh,'Legend');
axis(sh,'off');
sh=subplot(2,4,8);
leg2=legend(sh,plot1(floor(p/2+1):p),s{floor(p/2+1):p});
set(leg2,'position', get(sh,'position'));
axis(sh,'off');

width=800;
height=480;
set(fig, 'Position', [0 0 width+1 height+1]);